
SCREEN_RAM			= $1E00
COLOR_RAM			= $9600
CHARACTER_RAM		= $1C00

VICCR3				= $9003		; Raster scan line, high order
VICCR4				= $9004		; Raster scan line, low order
VICCR5				= $9005		; Screen map and Character Map Location
VICCRF				= $900F		; Background and Border Color
BLACK_BORDER_BLACK_BACKGROUND = $0008

JIFFYCLOCK			= $A2
JIFFYCOUNT			= $0A

FREKZP				= $FB

SCREEN_PAGE			= $288
SCREEN_ADDRESS		= FREKZP
COLOR_ADDRESS		= FREKZP + 2
SPRITE_ADDRESS		= FREKZP + 2

DISKDRIVE			= 8	; disk drive
FILENUMBER			= 1
SETLFS				= $FFBA
SETNAME				= $FFBD
LOADFILE			= $FFD5

SPACE				= $20

BLACK				= $00
WHITE				= $01
RED					= $02
CYAN				= $03
PURPLE				= $04
GREEN				= $05
BLUE				= $06
YELLOW				= $07

SPRITE_POSITION_V	= $00
SPRITE_POSITION_H	= $01
SPRITE_CHARACTER	= $02
SPRITE_COLOR		= $03
SPRITE_CLEAR_CHAR	= $04
SPRITE_DIRECTION_V	= $05
SPRITE_DIRECTION_H	= $06
SPRITE_HOME_V		= $07
SPRITE_HOME_H		= $08
SPRITE_SPEED		= $09
SPRITE_MOVE_COUNTER	= $0A

GHOST_MODE			= $0B
GHOST_TARGET_V		= $0C
GHOST_TARGET_H		= $0D
GHOST_SCATTER_V		= $0E
GHOST_SCATTER_H		= $0F
GHOST_DEFAULT_COLOR	= $10

MODE_SCATTER		= $00	; Ghosts target their scatter square
MODE_HUNT			= $01	; Ghosts target is based on PacMan's POSITION
MODE_RUN			= $02	; Ghost is blue, and running from PacMan
MODE_HOME			= $03	; Ghost has been eaten, and is running home

MODE_CHANGE			= $40	; Number of frames before initiating a mode change


NO_DIRECTION		= $00
DOWN				= $01
RIGHT				= $01
UP					= $FF
LEFT				= $FF

DOT					= $0E
POWER_PELLET		= $0F

CURRENT_KEY			= $C5
KEY_NONE			= $40
KEY_UP				= $09	; Up
KEY_LEFT			= $11	; Left
KEY_DOWN			= $29	; Down
KEY_RIGHT			= $12	; Right
KEY_QUIT			= $30	; Quit
KEY_H				= $2B	; H

	.org $1001

	; BASIC program to boot the machine language code
	; 10 print Clear Screen
	; 20 SYS 4114
	.byte	$0A, $10, $0A, $00 
	.byte	$99, $22, $93, $22 
	.byte	$00, $14, $10, $14 
	.byte	$00, $9E, $34, $31 
	.byte	$31, $38, $00, $00, $00

	LDA VICCR5							; VIC Chip Register 5
	AND #$F0							; the high nibble contains the screen map address, this mask protects it
	ORA #$0F							; This Value sets the character map address to 1C00
	STA VICCR5

	LDA #BLACK_BORDER_BLACK_BACKGROUND
	STA VICCRF							; Set VIC Chip Register F, controls screen and border color

	; Load the color data directly into the color map
	LDA #FILENUMBER						; file number
	LDX #DISKDRIVE						; device number
	LDY #$FF							; open command
	JSR SETLFS							; set LFS
	
	LDA #colorFileNameX - colorFileName	; length of filename
	LDX #<colorFileName					; filename address
	LDY #>colorFileName					; filename address
	JSR SETNAME
	
	LDA #0
	LDX #$FF
	LDY #$FF
	JSR LOADFILE

	; Load the maze directly into the screen
	LDA #FILENUMBER						; file number
	LDX #DISKDRIVE						; device number
	LDY #$FF							; open command
	JSR SETLFS							; set LFS
	
	LDA #screenFileNameX - screenFileName	; length of filename
	LDX #<screenFileName				; filename address
	LDY #>screenFileName				; filename address
	JSR SETNAME
	
	LDA #0
	LDX #$FF
	LDY #$FF
	JSR LOADFILE

	gameLoop:
		
		JSR drawSprites

		; Synchronize the frame rate to the Jiffy Clock, after the sprites are drawn to the screen
		LDA #$00
		STA JIFFYCLOCK		; reset the clock

	syncFrame:
		LDA CURRENT_KEY		; since we might sit here for a quarter second, read the keyboard here
		STA lastKey
		CMP #KEY_QUIT
		BEQ done
		
		CMP #KEY_H
		BNE notH
		
		LDA #MODE_HOME
		STA inky + GHOST_MODE

		LDA #WHITE
		STA inky + SPRITE_COLOR
		
		notH:
		
		CLC
		LDA JIFFYCLOCK
		CMP #JIFFYCOUNT
		BCC syncFrame		; If less than

	rasterSync:
		LDA VICCR4
		BNE rasterSync		; Wait for the raster scan to reach the top to eliminate flicker

	frame:	
		INC frameCounter

		JSR checkCollisions
		JSR processDirectionKeys
		JSR clearSprites
		JSR movePacMan
		JSR modeGhosts
		JSR targetGhosts
		JSR moveGhosts
		JSR animateSprites
		
		JSR drawScore
		
		JMP gameLoop
		
done:	RTS					; End of game loop

;**************************************************************************************************
;*** Sub Routines	
;**************************************************************************************************

;**************************************************************************************************
;*** Draw Score
;**************************************************************************************************
drawScore:

	LDX #$16
	JSR setRowPointer

	LDX lives
	LDY #$00
	
	drawLife:

		LDA #$23
		STA (SCREEN_ADDRESS), Y
		LDA #YELLOW
		STA (COLOR_ADDRESS), Y
		INY
		DEX
		BNE drawLife

	LDY #$12

	LDA score
	LSR
	LSR
	LSR
	LSR
	STA (SCREEN_ADDRESS), Y
	LDA #WHITE
	STA (COLOR_ADDRESS), Y

	INY
	LDA score
	AND #$0F
	STA (SCREEN_ADDRESS), Y
	LDA #WHITE
	STA (COLOR_ADDRESS), Y

	INY
	LDA score + 1
	LSR
	LSR
	LSR
	LSR
	STA (SCREEN_ADDRESS), Y
	LDA #WHITE
	STA (COLOR_ADDRESS), Y

	INY
	LDA score + 1
	AND #$0F
	STA (SCREEN_ADDRESS), Y
	LDA #WHITE
	STA (COLOR_ADDRESS), Y

	RTS


;**************************************************************************************************
;*** 
;**************************************************************************************************
checkCollisions:

	LDA #<blinky
	STA SPRITE_ADDRESS
	LDA #>blinky
	STA SPRITE_ADDRESS + 1
	JSR checkCollision

	LDA #<pinky
	STA SPRITE_ADDRESS
	LDA #>pinky
	STA SPRITE_ADDRESS + 1
	JSR checkCollision

	LDA #<inky
	STA SPRITE_ADDRESS
	LDA #>inky
	STA SPRITE_ADDRESS + 1
	JSR checkCollision

	LDA #<clyde
	STA SPRITE_ADDRESS
	LDA #>clyde
	STA SPRITE_ADDRESS + 1
	JSR checkCollision

	RTS

checkCollision:

		LDY #SPRITE_POSITION_V
		LDA (SPRITE_ADDRESS), Y
		CMP pacMan + SPRITE_POSITION_V
		BNE exitCheckCollision

		LDY #SPRITE_POSITION_H
		LDA (SPRITE_ADDRESS), Y
		CMP pacMan + SPRITE_POSITION_H
		BNE exitCheckCollision
		
		LDY #GHOST_MODE
		LDA (SPRITE_ADDRESS), Y
		CMP #MODE_HOME					; Ghost is on the way home, he can't be hurt, or hurt
		BEQ exitCheckCollision
		
		CMP #MODE_RUN
		BNE checkCollisionPacMan		; Ghost eats PacMan
	
		SED       						; Decimal mode (BCD addition: NUM3 = NUM1 + NUM2)
		CLC
		LDA score						; Add low bytes first
		ADC #02
		STA score
		CLD

		LDY #GHOST_MODE
		LDA #MODE_HOME
		STA (SPRITE_ADDRESS), Y

		LDY #SPRITE_COLOR
		LDA #WHITE
		STA (SPRITE_ADDRESS), Y

		RTS

checkCollisionPacMan:

exitCheckCollision: RTS


newModeCounter:	.byte $00

modeGhosts:

	DEC modeCounter
	LDA modeCounter
	STA newModeCounter
	
	LDA #<blinky
	STA SPRITE_ADDRESS
	LDA #>blinky
	STA SPRITE_ADDRESS + 1
	JSR modeGhost

	LDA #<pinky
	STA SPRITE_ADDRESS
	LDA #>pinky
	STA SPRITE_ADDRESS + 1
	JSR modeGhost

	LDA #<inky
	STA SPRITE_ADDRESS
	LDA #>inky
	STA SPRITE_ADDRESS + 1
	JSR modeGhost

	LDA #<clyde
	STA SPRITE_ADDRESS
	LDA #>clyde
	STA SPRITE_ADDRESS + 1
	JSR modeGhost
	
	LDA newModeCounter
	STA modeCounter
	

modeGhostsReturn: RTS

;**************************************************************************************************
;*** 
;**************************************************************************************************
modeGhost:
	
	LDY #GHOST_MODE
	LDA (SPRITE_ADDRESS), Y
	CMP #MODE_HOME
	BNE modeGhostCheckPowerPellet
	
		; The ghost has been eaten, and is now in Home Mode, 
		; The ghost will remain in Home Mode until it reaches home
		; Nothing else will change that
		; TODO when the mode changes to home, change the ghost's color to white
		LDY #SPRITE_POSITION_V
		LDA (SPRITE_ADDRESS), Y		; Load Vertical Position
		LDY #SPRITE_HOME_V
		CMP (SPRITE_ADDRESS), Y		; Compare it to the Home Row
		BNE modeGhostReturn			; If it's not home, the mode remains MODE_HOME
	
		LDY #SPRITE_POSITION_H
		LDA (SPRITE_ADDRESS), Y		; Load Horizontal Position
		LDY #SPRITE_HOME_H
		CMP (SPRITE_ADDRESS), Y		; Compare it to the Home Column
		BNE modeGhostReturn			; If it's not home, the mode remains MODE_HOME
		
		LDA #MODE_HUNT				; The ghost has arrived at home, change the mode to hunt - to seek revenge
		LDY #GHOST_MODE
		STA (SPRITE_ADDRESS), Y

		LDY #GHOST_DEFAULT_COLOR	; Turn back to your normal color
		LDA (SPRITE_ADDRESS), Y
		LDY #SPRITE_COLOR
		STA (SPRITE_ADDRESS), Y
	
		RTS
	
	modeGhostCheckPowerPellet:
	
		LDX pacMan + SPRITE_POSITION_V
		LDY pacMan + SPRITE_POSITION_H
		JSR getChar
		CMP #POWER_PELLET				; is PacMan eating a Power Pellet?
		BNE modeGhostCheckCounter		; no
	
			; PacMan is on a power pellet, the ghost's mode should change to run
			LDA #MODE_RUN
			LDY #GHOST_MODE
			STA (SPRITE_ADDRESS), Y
		
			LDA #BLUE						; When running, the ghosts turn blue
			LDY #SPRITE_COLOR
			STA (SPRITE_ADDRESS), Y
			
			LDA #MODE_CHANGE
			STA newModeCounter				; Reset mode counter

			RTS
		
	modeGhostCheckCounter:				; We get here when mode is in (hunt, scatter, run)
	
		LDA modeCounter
		BNE modeGhostsReturn			; When counter is not zero return
		
		LDA #MODE_CHANGE
		STA newModeCounter				; Reset mode counter
	
		LDY #GHOST_MODE
		LDA (SPRITE_ADDRESS), Y
		CMP #MODE_SCATTER
		BNE modeGhostSetScatter
	
			LDA #MODE_HUNT				; If Mode is scatter, set it to hunt
			LDY #GHOST_MODE
			STA (SPRITE_ADDRESS), Y
		
			LDY #GHOST_DEFAULT_COLOR	; Turn back to your normal color
			LDA (SPRITE_ADDRESS), Y
			LDY #SPRITE_COLOR
			STA (SPRITE_ADDRESS), Y
		
			RTS
	
	modeGhostSetScatter:
		LDA #MODE_SCATTER				; If mode is run or hunt, set it to scatter
		LDY #GHOST_MODE
		STA (SPRITE_ADDRESS), Y
		
		LDY #GHOST_DEFAULT_COLOR	; Turn back to your normal color
		LDA (SPRITE_ADDRESS), Y
		LDY #SPRITE_COLOR
		STA (SPRITE_ADDRESS), Y
	
modeGhostReturn: RTS

;**************************************************************************************************
;***
;**************************************************************************************************
drawSprites:

	; Save the character that the ghost will overwrite
	LDX blinky + SPRITE_POSITION_V
	LDY blinky + SPRITE_POSITION_H
	JSR getChar
	STA blinky + SPRITE_CLEAR_CHAR

	LDX pinky + SPRITE_POSITION_V
	LDY pinky + SPRITE_POSITION_H
	JSR getChar
	STA pinky + SPRITE_CLEAR_CHAR
	
	LDX inky + SPRITE_POSITION_V
	LDY inky + SPRITE_POSITION_H
	JSR getChar
	STA inky + SPRITE_CLEAR_CHAR
	
	LDX clyde + SPRITE_POSITION_V
	LDY clyde + SPRITE_POSITION_H
	JSR getChar
	STA clyde + SPRITE_CLEAR_CHAR

	; Draw sprites
	LDX pacman + SPRITE_POSITION_V
	JSR setRowPointer
	LDY pacman + SPRITE_POSITION_H
	LDA pacman + SPRITE_CHARACTER
	STA (SCREEN_ADDRESS),Y
	LDA pacman + SPRITE_COLOR
	STA (COLOR_ADDRESS),Y

	LDX blinky + SPRITE_POSITION_V
	JSR setRowPointer
	LDY blinky + SPRITE_POSITION_H
	LDA blinky + SPRITE_CHARACTER
	STA (SCREEN_ADDRESS),Y
	LDA blinky + SPRITE_COLOR
	STA (COLOR_ADDRESS),Y

	LDX pinky + SPRITE_POSITION_V
	JSR setRowPointer
	LDY pinky + SPRITE_POSITION_H
	LDA pinky + SPRITE_CHARACTER
	STA (SCREEN_ADDRESS),Y
	LDA pinky + SPRITE_COLOR
	STA (COLOR_ADDRESS),Y

	LDX inky + SPRITE_POSITION_V
	JSR setRowPointer
	LDY inky + SPRITE_POSITION_H
	LDA inky + SPRITE_CHARACTER
	STA (SCREEN_ADDRESS),Y
	LDA inky + SPRITE_COLOR
	STA (COLOR_ADDRESS),Y

	LDX clyde + SPRITE_POSITION_V
	JSR setRowPointer
	LDY clyde + SPRITE_POSITION_H
	LDA clyde + SPRITE_CHARACTER
	STA (SCREEN_ADDRESS),Y
	LDA clyde + SPRITE_COLOR
	STA (COLOR_ADDRESS),Y


	RTS
	
;**************************************************************************************************
;*** 
;**************************************************************************************************
clearSprites:

	LDX pacman + SPRITE_POSITION_V
	JSR setRowPointer
	LDY pacman + SPRITE_POSITION_H
	LDA pacman + SPRITE_CLEAR_CHAR
	STA (SCREEN_ADDRESS), Y
	LDA #WHITE
	STA (COLOR_ADDRESS), Y

	LDX blinky + SPRITE_POSITION_V
	JSR setRowPointer
	LDY blinky + SPRITE_POSITION_H
	LDA blinky + SPRITE_CLEAR_CHAR
	STA (SCREEN_ADDRESS), Y
	LDA #WHITE
	STA (COLOR_ADDRESS), Y

	LDX pinky + SPRITE_POSITION_V
	JSR setRowPointer
	LDY pinky + SPRITE_POSITION_H
	LDA pinky + SPRITE_CLEAR_CHAR
	STA (SCREEN_ADDRESS), Y
	LDA #WHITE
	STA (COLOR_ADDRESS), Y

	LDX inky + SPRITE_POSITION_V
	JSR setRowPointer
	LDY inky + SPRITE_POSITION_H
	LDA inky + SPRITE_CLEAR_CHAR
	STA (SCREEN_ADDRESS), Y
	LDA #WHITE
	STA (COLOR_ADDRESS), Y

	LDX clyde + SPRITE_POSITION_V
	JSR setRowPointer
	LDY clyde + SPRITE_POSITION_H
	LDA clyde + SPRITE_CLEAR_CHAR
	STA (SCREEN_ADDRESS), Y
	LDA #WHITE
	STA (COLOR_ADDRESS), Y

	RTS

;**************************************************************************************************
;*** 
;**************************************************************************************************
targetGhosts:

	LDA #<blinky			; targetGhost expects the address of blinky to be in SPRITE_ADDRESS (FREKZP + 2)
	STA SPRITE_ADDRESS
	LDA #>blinky
	STA SPRITE_ADDRESS + 1
	JSR targetGhost

	LDA #<pinky				; targetGhost expects the address of pinky to be in SPRITE_ADDRESS (FREKZP + 2)
	STA SPRITE_ADDRESS
	LDA #>pinky
	STA SPRITE_ADDRESS + 1
	JSR targetGhost

	LDA #<inky				; moveGhost expects the address of inky to be in SPRITE_ADDRESS (FREKZP + 2)
	STA SPRITE_ADDRESS
	LDA #>inky
	STA SPRITE_ADDRESS + 1
	JSR targetGhost

	LDA #<clyde				; targetGhost expects the address of clyde to be in SPRITE_ADDRESS (FREKZP + 2)
	STA SPRITE_ADDRESS
	LDA #>clyde
	STA SPRITE_ADDRESS + 1
	JSR targetGhost

	RTS

targetGhost:

	LDY #GHOST_MODE
	LDA (SPRITE_ADDRESS), Y				; Load the current Ghost Mode row into A
	
	targetGhostHunt:					; TODO each ghost should have his own hunt behavior
		CMP #MODE_HUNT
		BNE targetGhostHome
		
		; Blinky should target PacMan
		; Pinky targets two tiles in front of PacMan
		; Inky targets the 
		; Clyde targets PacMan until he is within 4 tiles of PacMan, then his target becomes his scatter tile
		
		
		LDA pacMan + SPRITE_POSITION_V
		LDY #GHOST_TARGET_V
		STA (SPRITE_ADDRESS), Y
		LDA pacMan + SPRITE_POSITION_H
		LDY #GHOST_TARGET_H
		STA (SPRITE_ADDRESS), Y
		
		RTS
	
	targetGhostHome:
		CMP #MODE_HOME
		BNE targetGhostRun

		LDY #SPRITE_HOME_V
		LDA (SPRITE_ADDRESS), Y
		LDY #GHOST_TARGET_V
		STA (SPRITE_ADDRESS), Y
		LDY #SPRITE_HOME_H
		LDA (SPRITE_ADDRESS), Y
		LDY #GHOST_TARGET_H
		STA (SPRITE_ADDRESS), Y

		RTS
		
	targetGhostRun:
		CMP #MODE_RUN
		BNE targetGhostScatter

		LDY #SPRITE_HOME_V				; TODO run and scatter shoul dhave different behavior
		LDA (SPRITE_ADDRESS), Y
		LDY #GHOST_TARGET_V
		STA (SPRITE_ADDRESS), Y
		LDY #SPRITE_HOME_H
		LDA (SPRITE_ADDRESS), Y
		LDY #GHOST_TARGET_H
		STA (SPRITE_ADDRESS), Y

	targetGhostScatter:
		LDY #GHOST_SCATTER_V
		LDA (SPRITE_ADDRESS), Y
		LDY #GHOST_TARGET_V
		STA (SPRITE_ADDRESS), Y
		
		LDY #GHOST_SCATTER_H
		LDA (SPRITE_ADDRESS), Y
		LDY #GHOST_TARGET_H
		STA (SPRITE_ADDRESS), Y

	RTS


;**************************************************************************************************
;*** 
;**************************************************************************************************
moveGhosts:

	LDA #<blinky			;moveGhost expects the address of blinky to be in SPRITE_ADDRESS (FREKZP + 2)
	STA SPRITE_ADDRESS
	LDA #>blinky
	STA SPRITE_ADDRESS + 1
	JSR moveGhost

	LDA #<pinky			;moveGhost expects the address of pinky to be in SPRITE_ADDRESS (FREKZP + 2)
	STA SPRITE_ADDRESS
	LDA #>pinky
	STA SPRITE_ADDRESS + 1
	JSR moveGhost

	LDA #<inky			;moveGhost expects the address of inky to be in SPRITE_ADDRESS (FREKZP + 2)
	STA SPRITE_ADDRESS
	LDA #>inky
	STA SPRITE_ADDRESS + 1
	JSR moveGhost

	LDA #<clyde			;moveGhost expects the address of clyde to be in SPRITE_ADDRESS (FREKZP + 2)
	STA SPRITE_ADDRESS
	LDA #>clyde
	STA SPRITE_ADDRESS + 1
	JSR moveGhost

	RTS

;**************************************************************************************************
;*** 
;**************************************************************************************************
animateSprites:
	
	LDA blinky + SPRITE_CHARACTER
	EOR #$03
	STA blinky + SPRITE_CHARACTER

	LDA pinky + SPRITE_CHARACTER
	EOR #$03
	STA pinky + SPRITE_CHARACTER

	LDA inky + SPRITE_CHARACTER
	EOR #$03
	STA inky + SPRITE_CHARACTER

	LDA clyde + SPRITE_CHARACTER
	EOR #$03
	STA clyde + SPRITE_CHARACTER

	animatePacMan:
		LDA pacMan + SPRITE_CHARACTER
		CMP #$23
		BNE animatePacManCenter
	
		LDA pacMan + SPRITE_DIRECTION_V
		CMP #UP
		BNE animatePacManDown
		
		LDA #$24
		STA pacMan + SPRITE_CHARACTER
		RTS
		
	animatePacManDown:
		CMP #DOWN
		BNE animatePacManLeft
		
		LDA #$26
		STA pacMan + SPRITE_CHARACTER
		RTS
	
	animatePacManLeft:
		LDA pacMan + SPRITE_DIRECTION_H
		CMP #LEFT
		BNE animatePacManRight
		
		LDA #$27
		STA pacMan + SPRITE_CHARACTER
		RTS

	animatePacManRight:
		CMP #RIGHT
		BNE animatePacManCenter
		
		LDA #$25
		STA pacMan + SPRITE_CHARACTER
		RTS

	animatePacManCenter:
		LDA  #$23
		STA	pacMan + SPRITE_CHARACTER

	RTS


movePacMan:

	; PacMan only moves once every so many frames
	; SPRITE_SPEED indicates how many frames each move takes
	; SPRITE_MOVE_COUNTER is incremented every frame
	; When SPRITE_MOVE_COUNTER is equal to SPRITE_SPEED the ghost moves
	;LDA pacMan + SPRITE_SPEED				; Load PacMan's speed into A
	;INC pacMan + SPRITE_MOVE_COUNTER		; increment the movement counter
	;CMP pacMan + SPRITE_MOVE_COUNTER		; compare the counter to the speed
	;BNE exitMovePacMan						; if the counter hasn't reached the speed, we won't move this ghost

	;LDA #$00
	;STA pacMan + SPRITE_MOVE_COUNTER		; reset the move counter

	LDA pacMan + SPRITE_POSITION_V
	CLC
	ADC pacMan + SPRITE_DIRECTION_V
	TAX
	
	LDA pacMan + SPRITE_POSITION_H
	CLC
	ADC pacMan + SPRITE_DIRECTION_H
	TAY
	
	JSR isBlocked
	BCC movePacManMove		; Not Blocked

	LDA #NO_DIRECTION		; Blocked, Stop PacMan
	STA pacMan + SPRITE_DIRECTION_V
	STA pacMan + SPRITE_DIRECTION_H

	movePacManMove:
		
		LDA pacMan + SPRITE_POSITION_V
		CLC
		ADC pacMan + SPRITE_DIRECTION_V
		STA pacMan + SPRITE_POSITION_V
		
		LDA pacMan + SPRITE_POSITION_H
		CLC
		ADC pacMan + SPRITE_DIRECTION_H
		STA pacMan + SPRITE_POSITION_H
		

		LDX pacMan + SPRITE_POSITION_V
		LDY pacMan + SPRITE_POSITION_H
		JSR getChar
		CMP #POWER_PELLET				; is PacMan eating a Power Pellet?
		BNE movePacManCheckDot			; no

			SED       						; Decimal mode (BCD addition: NUM3 = NUM1 + NUM2)
			CLC
			LDA score + 1
			ADC #$50
			STA score + 1
			LDA score
			ADC #$00
			STA score
			CLD


		movePacManCheckDot:
			LDX pacMan + SPRITE_POSITION_V
			LDY pacMan + SPRITE_POSITION_H
			JSR getChar
			CMP #DOT				; is PacMan eating a dot
			BNE exitMovePacMan			; no

				SED       						; Decimal mode (BCD addition: NUM3 = NUM1 + NUM2)
				CLC
				LDA score + 1
				ADC #$01
				STA score + 1
				LDA score
				ADC #$00
				STA score
				CLD
		
	
exitMovePacMan:	RTS

;**************************************************************************************************
;*** 
;***	pointer to the current ghost should be stored in SPRITE_ADDRESS (FREKZP + 2)
;**************************************************************************************************
moveGhost:

	; Each ghost only moves once every so many frames
	; SPRITE_SPEED indicates how many frames each move takes
	; SPRITE_MOVE_COUNTER is incremented every frame
	; When SPRITE_MOVE_COUNTER is equal to SPRITE_SPEED the ghost moves
	
	;LDY #SPRITE_MOVE_COUNTER
	;LDA (SPRITE_ADDRESS), Y		; Load the move counter
	;CLC
	;ADC #$01					; increment it
	;STA (SPRITE_ADDRESS), Y		; Put it back
	;LDY #SPRITE_SPEED
	;CMP (SPRITE_ADDRESS), Y		; compare the counter to the speed
	;BEQ moveGhostGhostNow
	
	;RTS							; if the counter hasn't reached the speed, we won't move this ghost
	
moveGhostGhostNow:
	;LDY #SPRITE_MOVE_COUNTER
	;LDA #$00
	;STA (SPRITE_ADDRESS), Y		; reset the move counter

moveGhostCheckUp;
	LDY #SPRITE_POSITION_V
	LDA (SPRITE_ADDRESS), Y		; Load the current row into A
	LDY #GHOST_TARGET_V
	CMP (SPRITE_ADDRESS), Y		; Compare against the target row
	BEQ	moveGhostCheckLeft		; it's on the same row
	BCC moveGhostCheckDown 		; Target Row is greater than A

	LDY #SPRITE_DIRECTION_V
	LDA (SPRITE_ADDRESS), Y		; Get current vertical direction
	CMP #DOWN					; Are we currently going down?
	BEQ moveGhostCheckDown		; We can't go backwards

	LDY #SPRITE_POSITION_V
	LDA (SPRITE_ADDRESS), Y		; Load the current row into A
	CLC
	ADC #UP						; subtract 1 from the current row
	TAX							; Row into X
	LDY #SPRITE_POSITION_H
	LDA (SPRITE_ADDRESS), Y
	TAY							; Column into Y
	JSR isBlocked
	BCS moveGhostCheckDown		; Blocked

	LDA #UP						; new Vertical direction is UP
	LDY #SPRITE_DIRECTION_V
	STA (SPRITE_ADDRESS), Y		; Store vertical direction

	LDA #NO_DIRECTION			; Clear horizontal direction
	LDY #SPRITE_DIRECTION_H
	STA (SPRITE_ADDRESS), Y		; Store horizontal direction
	JMP moveGhostMove
	
	moveGhostCheckDown:
		LDY #SPRITE_POSITION_V
		LDA (SPRITE_ADDRESS), Y		; Load the current row into A
		LDY #GHOST_TARGET_V
		CMP (SPRITE_ADDRESS), Y		; Compare against the target row
		BEQ	moveGhostCheckLeft		; it's on the same row
		BCS moveGhostCheckLeft 		; Target Row is less than A

		LDY #SPRITE_DIRECTION_V
		LDA (SPRITE_ADDRESS), Y		; Get current vertical direction
		CMP #UP						; We're currently going up, 
		BEQ moveGhostCheckLeft		; we can't go backwards

		LDY #SPRITE_POSITION_V
		LDA (SPRITE_ADDRESS), Y		; Load the current row into A
		CLC
		ADC #DOWN					; add 1 to the current row
		TAX							; Row into X
		LDY #SPRITE_POSITION_H
		LDA (SPRITE_ADDRESS), Y
		TAY							; Load the current column into Y
		JSR isBlocked
		BCS moveGhostCheckLeft		; Blocked

		LDA #DOWN					; New Vertical Direction is DOWN
		LDY #SPRITE_DIRECTION_V
		STA (SPRITE_ADDRESS), Y		; Store vertical direction

		LDA #NO_DIRECTION			
		LDY #SPRITE_DIRECTION_H
		STA (SPRITE_ADDRESS), Y		; Clear horizontal direction
		JMP moveGhostMove

	moveGhostCheckLeft:
		LDY #SPRITE_POSITION_H
		LDA (SPRITE_ADDRESS), Y		; Load the current column into A
		LDY #GHOST_TARGET_H
		CMP (SPRITE_ADDRESS), Y		; compare with the target column
		BEQ	moveGhostCheckCurrent	; it's on the same column
		BCC moveGhostCheckRight		; Target Column is greater than Current Column

		LDY #SPRITE_DIRECTION_H
		LDA (SPRITE_ADDRESS), Y		; Get current horizontal direction
		CMP #RIGHT					; We're currently going right, 
		BEQ moveGhostCheckRight		; we can't go backwards

		LDY #SPRITE_POSITION_V
		LDA (SPRITE_ADDRESS), Y
		TAX							; Current row into X
		LDY #SPRITE_POSITION_H
		LDA (SPRITE_ADDRESS), Y		; Load the current column into A
		CLC
		ADC #LEFT					; subtract 1
		TAY							; New column into Y
		JSR isBlocked
		BCS moveGhostCheckRight		; Blocked

		LDA #NO_DIRECTION			; Clear vertical direction
		LDY #SPRITE_DIRECTION_V
		STA (SPRITE_ADDRESS), Y

		LDA #LEFT					; Set norizontal direction to left
		LDY #SPRITE_DIRECTION_H
		STA (SPRITE_ADDRESS), Y
		JMP moveGhostMove

	moveGhostCheckRight:
		LDY #SPRITE_POSITION_H
		LDA (SPRITE_ADDRESS), Y		; Load the current column into A
		LDY #GHOST_TARGET_H
		CMP (SPRITE_ADDRESS), Y		; compare with the target column
		BEQ	moveGhostCheckCurrent	; it's on the same column
		BCS moveGhostCheckCurrent	; Target Column is less than Current Column

		LDY #SPRITE_DIRECTION_H
		LDA (SPRITE_ADDRESS), Y		; Get current horizontal direction
		CMP #LEFT					; We're currently going left, 
		BEQ moveGhostCheckCurrent	; we can't go backwards

		LDY #SPRITE_POSITION_V
		LDA (SPRITE_ADDRESS), Y
		TAX							; Current row into X
		LDY #SPRITE_POSITION_H
		LDA (SPRITE_ADDRESS), Y		; Load the current column into A
		CLC
		ADC #RIGHT					; add 1
		TAY							; New column into Y
		JSR isBlocked
		BCS moveGhostCheckCurrent	; Blocked

		LDA #NO_DIRECTION			; Clear vertical direction
		LDY #SPRITE_DIRECTION_V
		STA (SPRITE_ADDRESS), Y

		LDA #RIGHT					; Set Horizontal direction to left
		LDY #SPRITE_DIRECTION_H
		STA (SPRITE_ADDRESS), Y
		JMP moveGhostMove
		
	; We have to test a number of different directions
	newDirection_V:		.byte NO_DIRECTION
	newDirection_H:		.byte NO_DIRECTION
		
	moveGhostCheckCurrent:			; We wind up here if all paths to the target are currently blocked

		LDY #SPRITE_DIRECTION_V
		LDA (SPRITE_ADDRESS), Y
		STA newDirection_V			; Save the current vertical direction
		
		LDY #SPRITE_DIRECTION_H
		LDA (SPRITE_ADDRESS), Y
		STA newDirection_H			; Save the current horizontal direction

	moveGhostCheckNewDirection:
		LDY #SPRITE_POSITION_V
		LDA (SPRITE_ADDRESS), Y		; Load the current Row into A
		CLC
		ADC newDirection_V			; Add the vertical direction
		TAX							; New row into X
		LDY #SPRITE_POSITION_H
		LDA (SPRITE_ADDRESS), Y		; Load the current column into A
		CLC
		ADC newDirection_H			; Add horizontal direction
		TAY							; New column into Y
		JSR isBlocked
		BCS moveGhostCheckVertical	; Blocked

		;Square is not blocked
		LDA newDirection_V			; New Vertical Direction
		LDY #SPRITE_DIRECTION_V
		STA (SPRITE_ADDRESS), Y		; Store the new Vertical direction
		
		LDA newDirection_H			; New Horizontal Direction
		LDY #SPRITE_DIRECTION_H
		STA (SPRITE_ADDRESS), Y		; Store the new Horizontal direction

		JMP moveGhostMove

	moveGhostCheckVertical:
		LDY #SPRITE_DIRECTION_V
		LDA (SPRITE_ADDRESS), Y		; Load the current vertical direction
		
		BNE moveGhostCheckHorizontal	; We're currently moving vertically
										; The current direction is blocked
										; and We can't go backwards
										
		
		; We're currently moving horizontally
		; But, we're blocked and we can't move backwards
		LDA #NO_DIRECTION			; clear horizontal
		STA newDirection_H

		LDA newDirection_V
		CMP #UP
		BEQ moveGhostCheckVerticalDown	; We already tried up, let's try down

		LDA #UP						; let's try and go up
		STA newDirection_V
		JMP moveGhostCheckNewDirection
		
	moveGhostCheckVerticalDown:
		LDA #DOWN					; let's try and go DOWN
		STA newDirection_V
		JMP moveGhostCheckNewDirection

		
	moveGhostCheckHorizontal:

		; We're currently moving vertically
		; But, the current direction is blocked
		; and We can't go backwards
		LDA #NO_DIRECTION			; clear vertical
		STA newDirection_V
		
		LDA newDirection_H			; Have we tried LEFT yet?
		CMP #LEFT
		BEQ moveGhostCheckHorizontalRight

		LDA #LEFT					; Try to go LEFT
		STA newDirection_H
		JMP moveGhostCheckNewDirection
		
	moveGhostCheckHorizontalRight:	
		LDA #RIGHT					; Try to go RIGHT
		STA newDirection_H
		JMP moveGhostCheckNewDirection

	moveGhostMove:
		LDY #SPRITE_POSITION_V
		LDA (SPRITE_ADDRESS), Y		; Load the current row into A
		LDY #SPRITE_DIRECTION_V
		CLC
		ADC (SPRITE_ADDRESS), Y		; Add the vertical direction
		LDY #SPRITE_POSITION_V
		STA (SPRITE_ADDRESS), Y		; Save the new Row
		
		LDY #SPRITE_POSITION_H
		LDA (SPRITE_ADDRESS), Y		; Load the current column into A
		LDY #SPRITE_DIRECTION_H
		CLC
		ADC (SPRITE_ADDRESS), Y		; Add the horizontal direction
		LDY #SPRITE_POSITION_H
		STA (SPRITE_ADDRESS), Y		; Save the new Column

exitMoveGhost:	RTS

processDirectionKeys:

	LDA lastKey						; Load Last Key pressed into A
	
	processUp:
		CMP #KEY_UP						; Check for UP key pressed
		BNE processDown					; if not, test for Down key
		
		LDA pacMan + SPRITE_POSITION_V	; Load current Row
		CLC
		ADC #UP							; Subtract 1 from current row
		TAX								; Transfer result to X

		LDA pacMan + SPRITE_POSITION_H	; Load current Column
		TAY								; Transfer column into Y

		JSR isBlocked					; Check if that's blocked
		BCS noValidKey					; If it is, consider the key invalid
		
		LDA #UP							; Set Vertical Direction to Up
		STA pacMan + SPRITE_DIRECTION_V

		LDA #NO_DIRECTION				; Clear Horizontal direction
		STA pacMan + SPRITE_DIRECTION_H

		RTS

	processDown:
		CMP #KEY_DOWN					; Check for the Down key
		BNE processLeft					; If not, check for the Left key
		
		LDA pacMan + SPRITE_POSITION_V	; Load the current Row
		CLC
		ADC #DOWN						; Add 1 to it
		TAX								; Transfer result to X

		LDA pacMan + SPRITE_POSITION_H	; Load current column
		TAY								; Transfer that to Y

		JSR isBlocked					; Check if it's blocked
		BCS noValidKey					; If so, the key is not valid
		
		LDA #DOWN						; Set Vertical Direction to Down
		STA pacMan + SPRITE_DIRECTION_V
		
		LDA #NO_DIRECTION				; Clear Horizontal Direction
		STA pacMan + SPRITE_DIRECTION_H

		RTS

	processLeft:
		CMP #KEY_LEFT					; Check for the Left key
		BNE processRight				; If not, check for the Right key

		LDA pacMan + SPRITE_POSITION_V	; Load the current Row
		TAX								; Transfer it to X
		
		LDA pacMan + SPRITE_POSITION_H	; Load the current Column
		CLC
		ADC #LEFT						; Subtract One from it
		TAY								; Transfer result to Y
		
		JSR isBlocked					; Check if it's blocked
		BCS noValidKey					; If so, the key is not valid
		
		LDA #NO_DIRECTION				; Clear Vertical Direction
		STA pacMan + SPRITE_DIRECTION_V
		
		LDA #LEFT						; Set Horizontal Direction to Left
		STA pacMan + SPRITE_DIRECTION_H

		
		RTS

	processRight:
		CMP #KEY_RIGHT					; Check for the Right key
		BNE noValidKey					; If not, No Valid Key was Pressed
		
		LDA pacMan + SPRITE_POSITION_V	; Load the current Row
		TAX								; Transfer it to X
		
		LDA pacMan + SPRITE_POSITION_H	; Load the current Column
		CLC
		ADC #RIGHT						; Add One to it
		TAY								; Transfer result to Y
		
		JSR isBlocked					; Check if it's blocked
		BCS noValidKey					; If so, the key is not valid
		
		LDA #NO_DIRECTION				; Clear Vertical Direction
		STA pacMan + SPRITE_DIRECTION_V
		
		LDA #RIGHT						; Set Horizontal Direction to Right
		STA pacMan + SPRITE_DIRECTION_H
		
		
		RTS

noValidKey:
	RTS



;**************************************************************************************************
;*** setRowPointer takes the row value in X and adds 22 to the screen and color addresses in zero page, X times
;*** This leaves the screenAddress and colorAddress in zero page, pointing at the first column of the X row
;*** Then the column can be loaded into Y to use indirect Y to address the screen and color location
;**************************************************************************************************
setRowPointer:
	; store the base screen addresses in zero page, for indirect-indexed addressing
	LDA #$00
	STA SCREEN_ADDRESS
	STA COLOR_ADDRESS
	LDA SCREEN_PAGE
	STA SCREEN_ADDRESS + 1
	; convert screen address to color address
	AND #$03	
	ORA #$94
	STA COLOR_ADDRESS + 1

	setRowPointerNext:
		CLC
		LDA SCREEN_ADDRESS
		ADC #$16
		STA SCREEN_ADDRESS
		LDA SCREEN_ADDRESS + 1
		ADC #$00
		STA SCREEN_ADDRESS + 1

		CLC
		LDA COLOR_ADDRESS
		ADC #$16
		STA COLOR_ADDRESS
		LDA COLOR_ADDRESS + 1
		ADC #$00
		STA COLOR_ADDRESS + 1

		DEX
		BNE setRowPointerNext
	RTS
	
;**************************************************************************************************
;*** Takes the row, in X, and the column, in Y, finds the character at that location and returns it in A
;**************************************************************************************************
getChar:
	LDA #$00
	STA SCREEN_ADDRESS
	LDA SCREEN_PAGE
	STA SCREEN_ADDRESS + 1

	getCharRowNext:
		CLC
		LDA SCREEN_ADDRESS
		ADC #$16
		STA SCREEN_ADDRESS
		LDA SCREEN_ADDRESS + 1
		ADC #$00
		STA SCREEN_ADDRESS + 1

		DEX
		BNE getCharRowNext

		LDA (SCREEN_ADDRESS), Y
	RTS

;**************************************************************************************************
;*** Takes the row, in X, and the column, in Y, finds the character
;*** Sets the Carry Flag if the character is blocked
;**************************************************************************************************
isBlocked:
	JSR getChar 				; loads a with the character at row (X), col (Y)
	AND #$F0
	CMP #$10					; All maze characters are in the range $10 - $1F
	BEQ isBlockedTrue			; square is blocked
	CLC							; Not blocked, clear the carry flag
	RTS
	isBlockedTrue:
		SEC							; Blocked, set Carry

	RTS


;**************************************************************************************************
;*** variables
;**************************************************************************************************
lastKey:		.byte KEY_NONE
frameCounter:	.byte $00
modeCounter:	.byte MODE_CHANGE
runCounter:		.byte $00

lives:			.byte $03
score:			.byte $00, $00

;**************************************************************************************************
;*** Strings	
;**************************************************************************************************
screenFileName:
	;     s    c    r    e    e    n    d    a    t    a
	.byte $53, $43, $52, $45, $45, $4E, $44, $41, $54, $41
screenFileNameX:

colorFileName:
	;     c    o    l    o    r    d    a    t    a
	.byte $43, $4F, $4C, $4F, $52, $44, $41, $54, $41
colorFileNameX:

;**************************************************************************************************
;*** Sprites	
;**************************************************************************************************
pacMan:
	.byte $10, $0A							; SPRITE_POSITION_V, SPRITE_POSITION_H
	.byte $23, YELLOW						; SPRITE_CHARACTER, SPRITE_COLOR
	.byte SPACE								; SPRITE_CLEAR_CHAR
	.byte NO_DIRECTION, NO_DIRECTION		; SPRITE_DIRECTION_V, SPRITE_DIRECTION_H
	.byte $10, $0A							; SPRITE_HOME_V, SPRITE_HOME_H
	.byte $04, $00							; SPRITE_SPEED, SPRITE_MOVE_COUNTER

;ghosts
blinky:
	.byte $0B, $0A							; SPRITE_POSITION_V, SPRITE_POSITION_H
	.byte $21, RED							; SPRITE_CHARACTER, SPRITE_COLOR
	.byte SPACE								; SPRITE_CLEAR_CHAR
	.byte UP, NO_DIRECTION					; SPRITE_DIRECTION_V, SPRITE_DIRECTION_H
	.byte $0B, $0A							; SPRITE_HOME_V, SPRITE_HOME_H
	.byte $04, $00							; SPRITE_SPEED, SPRITE_MOVE_COUNTER

	.byte MODE_SCATTER						; GHOST_MODE
	.byte $03, $02							; GHOST_TARGET_V, GHOST_TARGET_H
	.byte $03, $02							; GHOST_SCATTER_V, GHOST_SCATTER_H
	.byte RED								; GHOST_DEFAULT_COLOR
	
pinky:
	.byte $0B, $0B							; SPRITE_POSITION_V, SPRITE_POSITION_H
	.byte $21, PURPLE						; SPRITE_CHARACTER, SPRITE_COLOR
	.byte SPACE								; SPRITE_CLEAR_CHAR
	.byte UP, NO_DIRECTION					; SPRITE_DIRECTION_V, SPRITE_DIRECTION_H
	.byte $0B, $0B							; SPRITE_HOME_V, SPRITE_HOME_H
	.byte $06, $00							; SPRITE_SPEED, SPRITE_MOVE_COUNTER

	.byte MODE_SCATTER						; GHOST_MODE
	.byte $03, $13							; GHOST_TARGET_V, GHOST_TARGET_H
	.byte $03, $13							; GHOST_SCATTER_V, GHOST_SCATTER_H
	.byte PURPLE							; GHOST_DEFAULT_COLOR

inky:
	.byte $0B, $0A							; SPRITE_POSITION_V, SPRITE_POSITION_H
	.byte $21, CYAN							; SPRITE_CHARACTER, SPRITE_COLOR
	.byte SPACE								; SPRITE_CLEAR_CHAR
	.byte DOWN, NO_DIRECTION				; SPRITE_DIRECTION_V, SPRITE_DIRECTION_H
	.byte $0B, $0A							; SPRITE_HOME_V, SPRITE_HOME_H
	.byte $06, $00							; SPRITE_SPEED, SPRITE_MOVE_COUNTER

	.byte MODE_SCATTER						; GHOST_MODE
	.byte $13, $02							; GHOST_TARGET_V, GHOST_TARGET_H
	.byte $13, $02							; GHOST_SCATTER_V, GHOST_SCATTER_H
	.byte CYAN								; GHOST_DEFAULT_COLOR

clyde:
	.byte $0B, $0B							; SPRITE_POSITION_V, SPRITE_POSITION_H
	.byte $21, GREEN						; SPRITE_CHARACTER, SPRITE_COLOR
	.byte SPACE								; SPRITE_CLEAR_CHAR
	.byte DOWN, NO_DIRECTION				; SPRITE_DIRECTION_V, SPRITE_DIRECTION_H
	.byte $0B, $0B							; SPRITE_HOME_V, SPRITE_HOME_H
	.byte $08, $00							; SPRITE_SPEED, SPRITE_MOVE_COUNTER

	.byte MODE_SCATTER						; GHOST_MODE
	.byte $13, $13							; GHOST_TARGET_V, GHOST_TARGET_H
	.byte $13, $13							; GHOST_SCATTER_V, GHOST_SCATTER_H
	.byte GREEN								; GHOST_DEFAULT_COLOR

;**************************************************************************************************
;*** custom character set	
;**************************************************************************************************
	.org CHARACTER_RAM
	.byte	$3C, $42, $46, $5A, $62, $42, $3C, $00	; 0			$00
	.byte	$08, $18, $28, $08, $08, $08, $3E, $00	; 1			$01
	.byte	$3C, $42, $02, $0C, $30, $40, $7E, $00	; 2			$02
	.byte	$3C, $42, $02, $1C, $02, $42, $3C, $00	; 3			$03
	.byte	$04, $0C, $14, $24, $7E, $04, $04, $00	; 4			$04
	.byte	$7E, $40, $78, $04, $02, $44, $38, $00	; 5			$05
	.byte	$1C, $20, $40, $7C, $42, $42, $3C, $00	; 6			$06
	.byte	$7E, $42, $04, $08, $10, $10, $10, $00	; 7			$07
	.byte	$3C, $42, $42, $3C, $42, $42, $3C, $00	; 8			$08
	.byte	$3C, $42, $42, $3E, $02, $04, $38, $00	; 9			$09
	.byte	$00, $00, $00, $00, $00, $00, $00, $00	; 			$0A
	.byte	$00, $00, $00, $00, $00, $00, $00, $00	; 			$0B
	.byte	$00, $00, $00, $00, $00, $00, $00, $00	; 			$0C
	.byte	$00, $00, $00, $00, $00, $00, $00, $00	; 			$0D
	.byte	$00, $00, $00, $18, $18, $00, $00, $00	; dot		$0E
	.byte	$00, $00, $18, $3C, $3C, $18, $00, $00	; power		$0F
	.byte	$FF, $FF, $00, $00, $00, $00, $FF, $FF	; maze		$10
	.byte	$C3, $C3, $C3, $C3, $C3, $C3, $C3, $C3	; maze		$11
	.byte	$3C, $7E, $E7, $C3, $C3, $C3, $C3, $C3	; maze		$12
	.byte	$FC, $FE, $07, $03, $03, $07, $FE, $FC	; maze		$13
	.byte	$C3, $C3, $C3, $C3, $C3, $E7, $7E, $3C	; maze		$14
	.byte	$3F, $7F, $E0, $C0, $C0, $E0, $7F, $3F	; maze		$15
	.byte	$C3, $C1, $C0, $C0, $C0, $E0, $7F, $3F	; maze		$16
	.byte	$3F, $7F, $E0, $C0, $C0, $C0, $C1, $C3	; maze		$17
	.byte	$FC, $FE, $07, $03, $03, $03, $83, $C3	; maze		$18
	.byte	$C3, $83, $03, $03, $03, $07, $FE, $FC	; maze		$19
	.byte	$C3, $81, $00, $00, $00, $00, $FF, $FF	; maze		$1A
	.byte	$C3, $C1, $C0, $C0, $C0, $C0, $C1, $C3	; maze		$1B
	.byte	$FF, $FF, $00, $00, $00, $00, $81, $C3	; maze		$1C
	.byte	$C3, $83, $03, $03, $03, $03, $83, $C3	; maze		$1D
	.byte	$C3, $81, $00, $00, $00, $00, $81, $C3	; maze		$1E
	.byte	$FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF	; BLOCK		$1F
	.byte	$00, $00, $00, $00, $00, $00, $00, $00	; SPACE		$20
	.byte	$3C, $7E, $DB, $DB, $FF, $FF, $FF, $AA	; ghost		$21
	.byte	$3C, $7E, $DB, $DB, $FF, $FF, $FF, $55	; ghost		$22
	.byte	$3C, $7E, $FF, $FF, $FF, $FF, $7E, $3C	; pacman	$23
	.byte	$24, $66, $E7, $FF, $FF, $FF, $7E, $3C	; pacman	$24
	.byte	$3C, $7E, $FF, $F8, $F8, $FF, $7E, $3C	; pacman	$25
	.byte	$3C, $7E, $FF, $FF, $FF, $E7, $66, $24	; pacman	$26
	.byte	$3C, $7E, $FF, $1F, $1F, $FF, $7E, $3C	; pacman	$27
	.byte	$18, $24, $42, $7E, $42, $42, $42, $00	; a			$28
	.byte	$7C, $22, $22, $3C, $22, $22, $7C, $00	; b			$29
	.byte	$1C, $22, $40, $40, $40, $22, $1C, $00	; c			$2A
	.byte	$78, $24, $22, $22, $22, $24, $78, $00	; d			$2B
	.byte	$7E, $40, $40, $78, $40, $40, $7E, $00	; e			$2C
	.byte	$7E, $40, $40, $78, $40, $40, $40, $00	; f			$2D
	.byte	$1C, $22, $40, $4E, $42, $22, $1C, $00	; g			$2E
	.byte	$42, $42, $42, $7E, $42, $42, $42, $00	; h			$2F
	.byte	$1C, $08, $08, $08, $08, $08, $1C, $00	; i			$30
	.byte	$0E, $04, $04, $04, $04, $44, $38, $00	; j			$31
	.byte	$42, $44, $48, $70, $48, $44, $42, $00	; k			$32
	.byte	$40, $40, $40, $40, $40, $40, $7E, $00	; l			$33
	.byte	$42, $66, $5A, $5A, $42, $42, $42, $00	; m			$34
	.byte	$42, $62, $52, $4A, $46, $42, $42, $00	; n			$35
	.byte	$18, $24, $42, $42, $42, $24, $18, $00	; o			$36
	.byte	$7C, $42, $42, $7C, $40, $40, $40, $00	; p			$37
	.byte	$18, $24, $42, $42, $4A, $24, $1A, $00	; q			$38
	.byte	$7C, $42, $42, $7C, $48, $44, $42, $00	; r			$39
	.byte	$3C, $42, $40, $3C, $02, $42, $3C, $00	; s			$3A
	.byte	$3E, $08, $08, $08, $08, $08, $08, $00	; t			$3B
	.byte	$42, $42, $42, $42, $42, $42, $3C, $00	; u			$3C
	.byte	$42, $42, $42, $24, $24, $18, $18, $00	; v			$3D
	.byte	$42, $42, $42, $5A, $5A, $66, $42, $00	; w			$3E
	.byte	$42, $42, $24, $18, $24, $42, $42, $00	; x			$3F
